#ifndef CMDALIAS_H
#define CMDALIAS_H
/*
   Definitions for command string aliases
   ======================================
   
   Copyright  2002 M.Kraemer, D-64291 Darmstadt, Seegartenstr. 32, 
                         all rights reserved

   $Log: cmdalias.h,v $
   Revision 1.1.2.1  2005/03/15 14:57:11  kraemer
   *** empty log message ***


*/

#include "u.h"
#include "cmdsys.h"
#include "cmderr.h"

#define CMDALIAS_DEBUG CMD_DEBUG
                                      /* a single alias entry */
struct SCMDALIASITEM {
   char *pcAlias;
   char *pcStr;                       /* command string to be aliased */
};
                                      /* structure for all alias activity */
struct SCMDALIAS {
   unsigned long ulFlag;              /* status flags */
   int nItem, mItem;                  /* current, max items */ 
   struct SCMDALIASITEM *psItem;
   struct SCMDLOCK sCL;
   struct SCMDERR sCE;                /* holds errors */
};
                                      /*  */ 
#define CMDALIAS_ACTIVE 0x00000001    /* alias objects are setup  and active */                                      
                                      /* slot increment  */ 
#define CMDALIAS_ITEMINC 8
                                      /* function specific flags */
/*
   _cmdaliasitemFree  : ( cmdaliasitem, flag )
                        o release  
                        o to test
  
   _cmdaliasDeleteItem: ( cmdalias, stritem, flag )
                        o delete specified entry in alias list 
                      8 x to test
                        o calls
                          o _cmdaliasitemFree()
  
   _cmdaliasFree      : ( cmdalias, flag )
                        o release resources 
                      3 x to test
                        o problem: shutdown when other tasks hold the lock
                        o calls:
                          o _cmdaliasitemFree()
                          o _cmdaliasInit()
  
   _cmdaliasGetItem   : ( cmdalias, cmdaliasitem, stralias, flag )
                        o locate item in list, copy w/ strdup()
                      7 x to test
                        o calls:
                          o _cmdaliasitemFree()
  
   _cmdaliasImportItem: ( cmdalias, cmdaliasitem, flag )
                        o store an entry in alias list 
                      5 x to test
                        o calls:
                          o _cmdaliasitemFree()
  
   _cmdaliasInit      : ( cmdalias, flag )
                        o initialize to "zero" state 
                      2 x to test
  
   _cmdaliasInspect   : ( cmdalias, flag )
                        o inspect  
                      1 x to test
  
   _cmdaliasList      : ( cmdalias, flag )
                        o list alias entries 
                      6 x to test
  
   _cmdaliasSetup     : ( cmdalias, flag )
                        o initialize for use 
                      4 x to test
                        o calls:
                          o _cmdaliasInit()
  
*/
                                      /* function prototypes */
int  _cmdaliasitemFree  ( struct SCMDALIASITEM *, unsigned long );
int  _cmdaliasDeleteItem( struct SCMDALIAS *, const char *, unsigned long );
int  _cmdaliasFree      ( struct SCMDALIAS *, unsigned long );
int  _cmdaliasGetItem   ( struct SCMDALIAS *, struct SCMDALIASITEM *, const char *, unsigned long );
int  _cmdaliasImportItem( struct SCMDALIAS *, struct SCMDALIASITEM *, unsigned long );
int  _cmdaliasInit      ( struct SCMDALIAS *, unsigned long );
int  _cmdaliasInspect   ( struct SCMDALIAS *, unsigned long );
int  _cmdaliasList      ( struct SCMDALIAS *, unsigned long );
int  _cmdaliasSetup     ( struct SCMDALIAS *, unsigned long );

struct SCMDALIAS *_cmdaliasTest( unsigned long );

#if defined(OLD)

#if defined(AMIGA)
#define CMDALIASLOCK( psCA )\
   ObtainSemaphore( &((psCA) -> sSigSem) );
#elif defined(__EMX__)
#define CMDALIASLOCK( psCA )\
   DosRequestMutexSem( (psCA) -> hmtx, SEM_INDEFINITE_WAIT );
#elif defined(CMDUNIX)
#define CMDALIASLOCK( psCA )\
   {\
   fcntl( psCA->iFile, F_SETLKW /*blocks*/, &(psCA->sLL) );\
   }
#else
#error You loose, you should not use this computer    
#endif


#if defined(AMIGA)
#define CMDALIASUNLOCK( psCA )\
   ReleaseSemaphore( &(psCA -> sSigSem) );
#elif defined(__EMX__)
#define CMDALIASUNLOCK( psCA )\
   DosReleaseMutexSem( psCA -> hmtx );
#elif defined(CMDUNIX)
#define CMDALIASUNLOCK( psCA )\
   {\
   fcntl( psCA->iFile, F_SETLKW /*blocks*/, &(psCA->sUL) );\
   }
#else
#error You loose, you should not use this computer       
#endif
#endif

int _cmdaliasError( struct SCMDERR *, unsigned long );

/*
   --- error codes
*/
enum { ERRCMDALIAS_SYS=1,
       ERRCMDALIAS_BADADDR, 
       ERRCMDALIAS_SYNTAX,
       ERRCMDALIAS_NOTFOUND,
       ERRCMDALIAS_BADNAME,
       ERRCMDALIAS_BADCMD,
};

#endif


