#ifndef CMDPRTCL_H
#define CMDPRTCL_H
/** \file
   Definitions for protocol stuff.
*/
/*
   
   Copyright  2002-2009 M.Kraemer, D-64291 Darmstadt, Seegartenstr. 32, 
                         all rights reserved

   $Log: cmdprtcl.h,v $
   Revision 1.1.2.1  2005/03/15 14:57:23  kraemer
   *** empty log message ***


*/

#if defined(__EMX__)
#define INCL_DOSSEMAPHORES
#include <os2.h>
#endif

#include "u.h"
#include "cmdsys.h"

                                      /** \struct SCMDPRTCL
                                          \brief Protocol stuff.
                                      */ 
struct SCMDPRTCL {
   unsigned long ulFlag;              /**< status flags */
   char *pcMsgFilter;                 /**< message filter string */
   int iFile;                         /**< protocol file handle */
#if defined(OLD)
#if defined(AMIGA)
   struct SignalSemaphore sSigSem;    /**< Exec Semaphore for protocol file locking */
#endif
#if defined(__EMX__)
   HMTX hmtx;                         /**< OS/2 Mutex for protocol file locking */
#endif
#if defined(CMDUNIX)
   struct flock sLL, sUL;             /**< protocol file locks */
#endif
#endif
   struct SCMDLOCK sCL;               /**< lock */
   char cFile[L_tmpnam+1];            /**< protocol file name */
};
                                      /* status flags */ 
#define CMDPRTCL_ACTIVE 0x00000001    /* objects are setup  and active */        
#define CMDPRTCL_ON     0x00000002    /* switch protocol on/off */
                                      /* function specific flags */
#define CMDPRTCL_NONE    0x00000000
#define CMDPRTCL_TERM    0x00010000   /* output terminal */
#define CMDPRTCL_FILE    0x00020000   /* output file */
#define CMDPRTCL_NOTIME  0x00100000   /* don't include time stamp */
#define CMDPRTCL_NOCOM   0x00200000   /* don't preceed with comment char */

/*
  
   _cmdprtcl            ( cmdprtcl, str, flag )
                        o output text to terminal and/or protocol file 
                      6 o to test
                          o A
                          o O 
                          x X
                        o calls
                          o _cmdprtclSetup
  
   _cmdprtclCopy2File   ( cmdprtcl, file, flag )
                        o snapshot current protocol to file 
                      7 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclDelete      ( cmdprtcl, flag )
                        o delete current protocol (do not free resources)
                      8 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclFree        ( cmdprtcl, flag )
                        o release resources 
                      3 o to test
                          o A
                          o O 
                          x X
                        o problem: shutdown when other tasks hold the lock
                        o calls:
                          o _cmdprtclInit()
  
  
   _cmdprtclImportMsgFilter( cmdprtcl, str, flag )
                        o set message filter 
                      5 o to test
                          o A
                          o O 
                          x X
                        o calls:
  
   _cmdprtclInit        ( cmdprtcl, flag )
                        o initialize to "zero" state 
                      2 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclInspect     ( cmdprtcl, flag )
                        o inspect  
                      1 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclOn/Off      ( cmdprtcl, flag )
                        o switch protocol on/off 
                      9 o to test
                          o A
                          o O 
                          x X
  
   _cmdprtclSetup       ( cmdprtcl, flag )
                        o initialize for use 
                      4 o to test
                          o A
                          o O 
                          x X
                        o calls:
                          o _cmdprtclInit()
                          o _cmdprtclFree()
  
*/
                                      /* function prototypes */
#if defined(OLD)
#if defined(AMIGA)
#define CMDPRTCLLOCK( psCP )\
   ObtainSemaphore( &((psCP) -> sSigSem) );
#elif defined(__EMX__)
#define CMDPRTCLLOCK( psCP )\
   DosRequestMutexSem( (psCP) -> hmtx, SEM_INDEFINITE_WAIT );
#elif defined(CMDUNIX)
#define CMDPRTCLLOCK( psCP )\
   {\
   fcntl( psCP->iFile, F_SETLKW /*blocks*/, &(psCP->sLL) );\
   }
#else
#error You loose, you should not use this computer    
#endif

#if defined(AMIGA)
#define CMDPRTCLUNLOCK( psCP )\
   ReleaseSemaphore( &(psCP -> sSigSem) );
#elif defined(__EMX__)
#define CMDPRTCLUNLOCK( psCP )\
   DosReleaseMutexSem( psCP -> hmtx );
#elif defined(CMDUNIX)
#define CMDPRTCLUNLOCK( psCP )\
   {\
   fcntl( psCP->iFile, F_SETLKW /*blocks*/, &(psCP->sUL) );\
   }
#else
#error You loose, you should not use this computer       
#endif
#endif
                                      /* function prototypes */
int  _cmdprtcl         ( struct SCMDPRTCL *, const char *, unsigned long );
int  _cmdprtclCopy2File( struct SCMDPRTCL *, const char *, unsigned long );
int  _cmdprtclDelete   ( struct SCMDPRTCL *, unsigned long );
int  _cmdprtclFree     ( struct SCMDPRTCL *, unsigned long );
int  _cmdprtclImportMsgFilter( struct SCMDPRTCL *, const char *, unsigned long );
int  _cmdprtclInit     ( struct SCMDPRTCL *, unsigned long );
int  _cmdprtclInspect  ( struct SCMDPRTCL *, unsigned long );
int  _cmdprtclOn       ( struct SCMDPRTCL *, unsigned long );
int  _cmdprtclOff      ( struct SCMDPRTCL *, unsigned long );
int  _cmdprtclSetup    ( struct SCMDPRTCL *, unsigned long );
int  _cmdprtclTest     ( struct SCMDPRTCL *, unsigned long );
void _prtcl            (       const char *, unsigned long );

#endif
