#ifndef IPAR_H
#define IPAR_H
/*
   Definitions for global parameters (IPARs)
   =================================

   Copyright  (C) 1992-2009 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved

   $Log: ipar.h,v $
   Revision 1.3.2.1  2005/03/15 14:57:26  kraemer
   *** empty log message ***

   Revision 1.3  2001/02/23 16:42:31  kraemer
   prototype bug fixed

   Revision 1.2  2000/04/03 12:38:25  kraemer
   from MC

*/


#include "u.h"
#include "m.h"

#include "mathtree.h"
#include "cmdsys.h"
#include "cmderr.h"
#include "cmdprtcl.h"
                                      /* masks */
#define IPAR_MASKGLOBAL 0xFF000000    /* global (e.g. debug) */
#define IPAR_MASKFUNC   0x00FF0000    /* function specific flags */
#define IPAR_MASKQUAL   0x0000FFFF    /* for additional qualifiers */
                                      /* global flags */
#define IPAR_DEBUG  0x80000000        /* general check flag */
#define IPAR_QUIET  0x10000000        /* suppress error messages */
#define IPAR_NOMSG  IPAR_QUIET

#define IPAR_E  M_E                   /* 2.71828182845904523536028747135266250 */  
#define IPAR_PI PI
#define IPAR_C  2.99792458E+08
#ifndef MFFP
#define IPAR_HBAR 1.0545887E-34
#define IPAR_NA   6.0220450E+23
#endif
#define IPAR_ALPHA 7.2973506E-03
#define IPAR_FSC   7.2973506E-03
#define IPAR_EV    1.6021892E-19
#define IPAR_AMU   9.315016E+02
#define IPAR_ME    5.110034E-01
#define IPAR_MN    9.395729E+02
#define IPAR_MP    9.382796E+02

                                      /* valid chars in global parameters */
#define ISIPARCHAR(c)  ( isalpha(c) || isdigit(c) || ( ((c) & 127) == 0x5f) || ( (c) == '$' ) || ( (c) == '#' ) )
                                      /* valid first chars in global parameters */
#define ISIPARCHARF(c) ( isalpha(c)               || ( ((c) & 127) == 0x5f) || ( (c) == '$' ) || ( (c) == '#' ) )
                                      /* valid chars in global parameters w/ wild cards */
#define ISIPARCHARWILD(c)  ( ISIPARCHAR(c)  || ( (c) == '*' ) || ( (c) == '%' ) )
                                      /* valid first chars in global parameters w/ wild cards */
#define ISIPARCHARWILDF(c) ( ISIPARCHARF(c) || ( (c) == '*' ) || ( (c) == '%' ) )

                                      /* help with string decomposition ... */
struct SIPARVALSTR {
   struct SMATHTREEVALSTR sMTVS;
   struct SCMDERR sCE;
};
 
/*

   _iparvalstrFree   ( ipvalstr, flag )
                   3 o release resources
                     x to test
                     o calls 
                       o _iparvalstrInit    
    
   _iparvalstrFromStr[n]( ipvalstr, str, [nstr,] flag )
                     o convert plain ASCII string to string values
                   4 x to test
                     x honour function flags
                     x honour ipar's string oddities
                     o calls
                       o _iparvalstrInit    
                       o _iparvalstrFree    
    
   _iparvalstrInit   ( ipvalstr, flag )
                     o initialize
                   2 x to test
                     o calls 
    
   _iparvalstrInspect( ipvalstr, flag )
                     o inspect
                   1 x to test
                     o calls 
    
*/ 
                                      /* function flags */
#define IPARVALSTR_COUNTONLY 0x00010000 
                                      /* function prototypes */
int _iparvalstrFree    ( struct SIPARVALSTR *, unsigned long );
int _iparvalstrFromStr ( struct SIPARVALSTR *, const char *, unsigned long );
int _iparvalstrFromStrn( struct SIPARVALSTR *, const char *, int, unsigned long );
int _iparvalstrInit    ( struct SIPARVALSTR *, unsigned long );
int _iparvalstrInspect ( struct SIPARVALSTR *, unsigned long );
struct SIPARVALSTR * _iparvalstrTest   ( unsigned long );


 
                                      /* format stuff for list, write */
#define IPARFMT_LINE 80

#define IPARFMT_BOOL   10
#define IPARFMT_SHORT  10
#define IPARFMT_LONG   10
#define IPARFMT_FLOAT  16 
#define IPARFMT_DOUBLE 24

#define IPARFMT_BOOLPERLINE   ( IPARFMT_LINE/(IPARFMT_BOOL+1) )
#define IPARFMT_SHORTPERLINE  ( IPARFMT_LINE/(IPARFMT_SHORT+1) )
#define IPARFMT_LONGPERLINE   ( IPARFMT_LINE/(IPARFMT_LONG+1) )
#define IPARFMT_FLOATPERLINE  ( IPARFMT_LINE/(IPARFMT_FLOAT+1) ) 
#define IPARFMT_DOUBLEPERLINE ( IPARFMT_LINE/(IPARFMT_DOUBLE+1) )

                                      /* help with string catenation */
struct SIPARSTR {
   char *pcStr; 
   int ncStr; 
   int ncBuf; 
};

#define IPARSTRCLR(ipstr) if ( (ipstr)->pcStr != NULL ) memset( (ipstr)->pcStr, 0, (ipstr)->ncStr );
#define IPARSTRINIT(ipstr) memset( ipstr, 0, sizeof(*(ipstr)) ); 
#define IPARSTRFREE(ipstr) if ( (ipstr)->pcStr != NULL ) free( (ipstr)->pcStr ); IPARSTRINIT(ipstr);  
#define IPARSTRCAT(ipstr,str,error)\
   if ( str != NULL ) {\
      (ipstr)->ncBuf = strlen(str)+1;\
      if ( (ipstr)->pcStr != NULL ) (ipstr)->ncBuf += strlen( (ipstr)->pcStr );\
      if ( (ipstr)->ncBuf > (ipstr) -> ncStr ) {\
         (ipstr)->pcStr = realloc( (ipstr)->pcStr, (ipstr)->ncBuf*sizeof(*((ipstr)->pcStr)) );\
         if ( (ipstr)->pcStr == NULL ) error;\
         (ipstr)->pcStr[(ipstr) -> ncStr] = 0;\
         (ipstr) -> ncStr = (ipstr)->ncBuf;\
      }\
      strcat( (ipstr)->pcStr, str);\
   }


                                      /* a single global parameter */
struct SIPAR {
   struct SMATHTREE sMT;
   unsigned long ulFlag;              /* flag bits */
   char *pcCom;                       /* comment */
   struct SCMDERR sCE;
};
                                      /* flag bits */
#define IPAR_REMOVABLE 0x00000001     /* created interactively */
#define IPAR_ISMATH    0x00000002     /* is a math/phys constant */

                                      /* function specific flags */
#define IPARFREE_ALL   0x00010000     /* on shutdown: remove protected ipar's too */

#define IPARLIST_MATHONLY 0x00010000  /* list math constants only */
#define IPARLIST_SHORT    0x00020000  /* short listing */
#define IPARLIST_NOREP    0x00040000  /* list w/o repetition factors */

#define IPARWRITE_NOREP 0x00040000    /* save w/o repetition factors */

#define IPARDCL_DYNAMIC 0x00010000    /* declare "dynamic" (i.e. removable) parameters */
#define IPARDCL_LIST    0x00020000    /* list created parameters */
#define IPARDCL_ISMATH  0x00040000    /* mark as math/phys constant */

#define IPARVALUESFROMSTR_SINGLE 0x00010000 /* assign single values to all ipars */

#define IPARFROMSTR_WILD   0x00010000 /* allow wildcarded names */
#define IPARFROMSTR_APPEND 0x00020000 /* don't initialize list, append items */
#define IPARFROMSTR_INTSUB 0x00040000 /* subscripts must be integer */

#define IPARREAD_QUIET      0x00020000/*  */
#define IPARREAD_SKIPVALUES 0x00040000/* don't read values, only names */

#define IPARDELETE_NOCONFIRM 0x00010000 /*  */

/*
   
   Single parameter handling functions
   -----------------------------------

   _iparDataReset            : ( ip, flag )
                             o reset data to zero
                             x to test
                             o calls 
                                  
   _iparDup                : ( ipto, ipfr, flag )
                             o duplicate ipar
                          4b x to test
                             o calls 
                               o _iparFree()
                               o _iparInit()
                                  
   _iparDupNoData            ( ipto, ipfr, flag )
                             o duplicate ipar header info
                          4a x to test
                             o calls 
                               o _iparFree()
                               o _iparInit()
                                  
   _iparFree               : ( ip, flag )
                             o release ipar resources
                           3 x to test
                             ? honour protection here or at caller ?
                             o calls 
                               o _iparInit()
                                  
   _iparImportComment      : ( ip, ccom, ncom, flag )
                             o import a comment string
                           5 x to test
                             x NULL deletes existing comment
                             ? how about delete-protected items 
                             o calls 
   
   _iparImportType         : ( ip, ctype, ntype, flag )
                             o import the data type
                           6 x to test
                             x reject if data already exists 
                             ? allocate data array, if possible/necessary
                             ? relegate allocation to mathtree*() ?
                             o calls 
   
   _iparImportValues       : ( ip, pvcValue[], nvcValue[], nval, ipsub, flag )
                             ( ip, ipvalstr, ipsub, flag )
                             o import values 
                           7 x to complete
                             x if no data space: allocate it now
                               x if no type: determine from values
                             o calls 
                               o _mathtreeImportValStrSub()
                               o _mathtreeSetImportSub()
      
   _iparInit               : ( ip, flag )
                             o initialize an ipar
                           2 x to test
                             o calls 
   
   _iparInspect            : ( ip, flag )
                             o inspect an ipar
                           1 x to test
                             o calls 
   
   _iparList               : ( ip, ipsub, flag )
                             o list an ipar, w/ optional subscripts 
                           8 o to complete:
                             o honour action flags
                             x listing layout
                               o short: name + subscript + type
                               o long : name + subscript + type 
                                        values
                             o commom w/ Write() ?           
                             o calls 
                               o _iparListFmt()
      
   _iparListFmt            : ( mathtree, flag )
                             o performs actual listing, recursively
                          8a o to complete
                               o repetition factors
                             o calls 
                               o _iparListFmt()
      
   _iparMatchAttr            ( ipto, ipfr, flag )
                             o returns 0 if ipar attributes match
                             o to do
                             o calls 
   
   _iparWrite              : ( ip, fp, flag )
                             o write specified ipar to given file
                           9 x to test
                             o calls 
                               o _iparWriteFmt()
      
   _iparWriteFmt           : ( fp, mathtree, flag )
                             o performs actual write on file, recursively
                          9a o to complete
                               o repetition factors
                             o calls 
                               o _iparWriteFmt()
      
*/                          

                                      /* function prototypes */
int _iparDataReset    ( struct SIPAR *, unsigned long );
int _iparDup          ( struct SIPAR *, struct SIPAR *, unsigned long );
int _iparDupNoData     ( struct SIPAR *, struct SIPAR *, unsigned long );
int _iparFree         ( struct SIPAR *, unsigned long );
int _iparImportComment( struct SIPAR *, const char *, int, unsigned long );
int _iparImportType   ( struct SIPAR *, const char *, int, unsigned long );
int _iparImportValues ( struct SIPAR *, struct SIPARVALSTR *, struct SIPAR *, unsigned long );
int _iparInit         ( struct SIPAR *, unsigned long );
int _iparInspect      ( struct SIPAR *, unsigned long );
int _iparList         ( struct SIPAR *, struct SIPAR *, unsigned long );
int _iparListFmt      ( struct SMATHTREE *, unsigned long );
int _iparMod          ( struct SIPAR *, struct SIPAR *, unsigned long );
int _iparWrite        ( struct SIPAR *, FILE *, unsigned long );
int _iparWriteFmt     ( FILE *, struct SMATHTREE *, unsigned long );
struct SIPAR * _iparTest( unsigned long );

                                      /* list header */
struct SIPARLIST {
   unsigned long ulFlag;              /* flag bits */
   struct List lh;
   struct SCMDLOCK sCL; 
};                

#define IPARLIST_ACTIVE 0x00000001

/*
   
   Parameter list handling functions
   ---------------------------------
  
   ? honour lock active/inactive
   ? how about sCE returns ? 
    
  
   _iparEvaluateFromStr      : ( ip, iplist, str, len, flag )
                             o try expression evaluation
                             x to test
                             o calls
                               o _iparlistLocate()  
                               o _iparInit()  
                               o _iparFree() 
                               o _mathtree*()
   
   
   
   _iparlistAdd            : ( iplist, ip, flag )
                             o add an entry to an ipar list, w/ duplicate
                           6 x to test
                             x duplicate mathtree w/o operands
                             x don't lock (must be done externally)
                             x don't initialize iplist
                             x how about already existing node ? 
                               => try delete, if possible
                             o calls
                               o _iparDup()
                               o _iparFree() 
                               o _iparlistLocate()
   
   _iparlistConst          : ( iplist, flag )
                             o define math/phys constants
                           9 x to test
                             x don't initialize iplist
                             x don't lock (must be done externally)
                             x lock, check existance, define as non-removeable/math, from internal table
                               => listdcl 
                             o calls 
                               o _iparlistDcl()
   
   _iparlistDataReset      : ( iplist, flag )
                             o reset data
                             x to test
                             o calls 
                               o _iparDataReset()
                                  
   _iparlistDcl            : ( iplist, names, type, init, comment, flag )
                             o declare a bunch of ipar's (IPARDCL, $PARDCL action)
                           8 x to test
                             x lock 
                             x check existence of single ipar's
                             x efficient looping
                             x initial values handling 
                             x honour function flags
                             o calls 
                               o _iparlistInit()
                               o _iparlistFree()
                               o _iparlistFromStr()
                               o _iparvalstr*()
                               o _iparImport*()
                               o _iparlistAdd()
                               o _iparFree()
                               o _iparvalstr*()
                               
   
   _iparlistFree           : ( iplist, flag )
                             o release an ipar list's resources
                           3 x to test
                             o calls 
                               o _iparlistInit()
                               o _iparFree()
                                  
   _iparlistFromStr        : ( iplist, cname, nname, flag )
                             o decompose list of ipar names (w/ subscripts),
                               initializes iplist
                           7 x to test
                             o calls 
                               o _iparlistAdd ()
                               o _iparlistInit()
                               o _iparlistFree()
                               o _iparInit()
                               o _iparFree()
                                  
   _iparlistInit           : ( iplist, flag )
                             o initialize an ipar list
                           2 x to test
                             o calls 
   
   _iparlistInspect        : ( iplist, flag )
                             o inspect an ipar list
                           1 x to test
                             o calls 
                               o _iparInspect()
   
   _iparlistIsWild           ( iplist, flag )
                             o check for wildcarded ipars in list
                             o to test
                             o calls 
                               o 
   
   _iparlistLocate         : ( iplist, name, flag )
                             o locate ipar by name, return struct SIPAR *
                           5 x to test
                             x don't lock
                             o calls 
   
   _iparlistSetup          : ( iplist, flag )
                             o setup an ipar list
                           4 x to test
                             o calls 
                               o _iparlistInit()




   _iparlistlistComment        ( iplist, iplistselect, comments, flag )
                               o import comments for selected parameters
                             4 x to test
                               x if active: lock
                               o calls:
                                 o _iparImportComment()
                             
   _iparlistlistDelete         ( iplist, iplistselect, flag )
                               o delete selected parameters
                             3 x to test
                               x if active: lock
                               x honour function flags
                               o calls
                                 o _iparFree()  
   
   _iparlistlistList           ( iplist, iplistselect, flag )
                               o list selected parameters
                             1 x to test
                               o account for subscripts 
                               o calls
                                 o _iparList() 
   
   _iparlistlistRead           ( iplist, iplistselect, pv, flag )
                               o read selected parameters from memory block
                             6 x to test
                               ! to complete
                                 o line catenation 
                                 x test subranges for non-existing targets
                                 x erase/rebuild targets if mismatch
                               ? create non-existing targets 
                                 or just skip them
                               x honour function flags  
                               o calls
                                 o _mathtree*()
                                 o _iparlistFree()
                                 o _iparlistInit()
                                 o _iparlistFromStr()
                                 o _iparlistLocate()
                                 o _iparDup()
                                 o _iparFree()
                                 o 
                                 
   _iparlistlistValuesFromStr  ( iplist, iplistselect, str, flag )
                               o fill selected ipar's w/ values
                             5 x to test
                               x if active: lock
                               ? w/ subscript selection    
                               o calls 
                                 o _iparImportValues()
                                 o _iparc2vcn()
                                 o _iparlistLocate(
                                 o _iparDup()
                                 o _iparFree()
    
   _iparlistlistWrite          ( iplist, iplistselect, fp, flag )
                               o write selected parameters 
                             2 x to test
                               x if active: lock
                               x honour function flags
                               o calls 
                                 o _iparWrite()
   
*/                          
                                      /* function action flags */

                                      /* function prototypes */
int _iparEvaluateFromStr( struct SIPAR *, struct SIPARLIST *, const char *, int, unsigned long );

int _iparlistAdd    ( struct SIPARLIST *, struct SIPAR *, unsigned long );
int _iparlistConst  ( struct SIPARLIST *, unsigned long );
int _iparlistDataReset( struct SIPARLIST *, unsigned long );
int _iparlistDcl    ( struct SIPARLIST *, const char *, const char *, const char *, const char *, unsigned long );
int _iparlistFree   ( struct SIPARLIST *, unsigned long );
int _iparlistFromStr( struct SIPARLIST *, const char *, int, unsigned long );
int _iparlistInit   ( struct SIPARLIST *, unsigned long ); 
int _iparlistInspect( struct SIPARLIST *, unsigned long ); 
int _iparlistIsWild ( struct SIPARLIST *, unsigned long ); 
int _iparlistList   ( struct SIPARLIST *, const char *, unsigned long ); 
struct SIPAR *_iparlistLocate( struct SIPARLIST *, const char *, unsigned long );
int _iparlistSetup  ( struct SIPARLIST *, unsigned long ); 

int _iparlistlistComment( struct SIPARLIST *, struct SIPARLIST *, const char *, unsigned long );
int _iparlistlistDelete ( struct SIPARLIST *, struct SIPARLIST *, unsigned long );
int _iparlistlistList   ( struct SIPARLIST *, struct SIPARLIST *, unsigned long );
int _iparlistlistRead   ( struct SIPARLIST *, struct SIPARLIST *, void *, unsigned long );
int _iparlistlistValuesFromStr( struct SIPARLIST *, struct SIPARLIST *, const char *, unsigned long );
int _iparlistlistWrite  ( struct SIPARLIST *, struct SIPARLIST *, FILE *, unsigned long );
 
struct SIPARLIST *_iparlistTest( unsigned long ); 

 
                                      /* error handling */
int _iparError( struct SCMDERR *, unsigned long );
                                      
enum {
   ERRIPAR_SYS=1,
   ERRIPAR_BADADDR,
   ERRIPAR_BADNAMES,
   ERRIPAR_BADNAME,
   ERRIPAR_BADLIST,
   ERRIPAR_NOTREMOVEABLE,
   ERRIPAR_BADTYPE,
   ERRIPAR_DATAEXISTS,
   ERRIPAR_BADINIT,
   ERRIPAR_BADCOMMENT,
   ERRIPAR_PARUNBALANCED,
   ERRIPAR_BADVALUES,
   ERRIPAR_BADCOUNT,
   ERRIPAR_DATAIMPORTMISMATCH,
   ERRIPAR_FILESYNTAX,
   ERRIPAR_VALUESYNTAX,
   ERRIPAR_BADREPETITION,
   ERRIPAR_QUOTESUNBALANCED,
   ERRIPAR_MAXDIM,
   ERRIPAR_LISTIDENTICAL,
   ERRIPAR_NOFILEDATA,
   ERRIPAR_BADSUBSCRIPT,
   ERRIPAR_NOTFOUND,
   ERRIPAR_NODATA,
   ERRIPAR_DIMMISMATCH,
   ERRIPAR_TYPEMISMATCH,
   ERRIPAR_BOUNDSMISMATCH,
   ERRIPAR_BADEXPR,
   ERRIPAR_MATHTREEFROMSTRFAILED,
   ERRIPAR_MATHTREEEVALUATEFAILED,
   ERRIPAR_MATHTREEDUPFAILED,
   ERRIPAR_MATHTREEDATAALLOCFAILED,
   ERRIPAR_MATHTREESETIMPORTSUBFAILED,
   ERRIPAR_MATHTREEIMPORTVALSTRSUBFAILED,
   ERRIPAR_MATHTREECOPYDATASUBFAILED,
   ERRIPAR_MATHTREECOLLECTUDFFAILED,
   ERRIPAR_MATHTREEIMPORTDATAFAILED,
};

#endif



